{------------------------------------------------------------------------------

Copyright (C) 2002  Rainer Queck
e-mail: Rainer.Queck@Qutronic.de
http://www.qutronic.de


This code is free; you can redistribute it and/or
modify it under the terms of the MPL MOZILLA PUBLIC LICENSE. 

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
MPL MOZILLA PUBLIC LICENSE for more details at http://www.mozilla.org/MPL

//-----------------------------------------------------------------------------}
// Unit Created at    : Date: 16.02.02  Time :13:17:46
// Unit last modified : Date: 17.04.03  Time :08:11:40
//------------------------------------------------------------------------------
unit ExpertInterface;

interface

uses Windows, SysUtils, Classes,stdctrls,menus,Forms,
     MMToolsApi,MMEngineDefs, Dialogs, QutronicBaseExpert, QutronicExpertClasses,
     InfoForm, QmmeMain, MailboxRq, AutoDiagramming;

type
  TQutronicExpertInterface = class (TMMBaseExpert)
  private
    FExpertFunctions: TExpertFunctions;
    FGui: TFrmMain;
    FMailbox: TRqMailBox;
  protected
    function GetMenuPositions(Index: Integer): TMMMenuPosition; override; 
            safecall;
    function GetMenuShortCuts(Index: Integer): TShortCut; override; safecall;
    function GetVerbCount: Integer; override; safecall;
    function GetVerbs(Index: Integer): WideString; override; safecall;
    procedure OnMbError(Sender : TObject ; Error : TMailBoxErrors); virtual;
    procedure OnMbReceive(Sender : TObject ; RxMessage : TMemoryStream); 
            virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute(Index: Integer); override; safecall;
    function ExpertID: WideString; override; safecall;
  end;
  

implementation

{
*************************** TQutronicExpertInterface ***************************
}
constructor TQutronicExpertInterface.Create;
begin
  inherited Create;
  
  FExpertFunctions := TExpertFunctions.Create;
  FGui := TFrmMain.Create(Nil);
  
  with TQutronicInfoFrm.Create(NIL) do
  begin
    Timer1.Interval:=1000;
    ShowModal;
    Free;
  end;
  FMailbox:=TRqMailbox.Create(False);
  FMailbox.BoxName:='MbQMME';
  FMailbox.OnError:=OnMbError;
  FMailbox.OnReceive:=OnMbReceive;
  FMailbox.Active:=True;
end;

destructor TQutronicExpertInterface.Destroy;
begin
  FExpertFunctions.Free;
  FGui.Free;
  
  FMailbox.Free;
  inherited Destroy;
end;

procedure TQutronicExpertInterface.Execute(Index: Integer);
var
  currClassMember: string;
  aRefList: TStringList;
  
  const
    aClassName : String='';
  
begin
  case Index of
    0: begin
         FExpertFunctions.AddHeader;
      end;
    1: begin
         FExpertFunctions.ApplyTryExceptToCurrentMethod;
       end;
    2: begin
         FGui.Show;
       end;
    3: begin
         FGui.Show;
         FGui.PgctrlMain.ActivePage:=FGui.tbsReference;
         FGui.SpBtSearchHyperLinksCurrentDiagramClick(self);
       end;
    4: begin
         FGui.Show;
         FGui.PgctrlMain.ActivePage:=FGui.tbsReference;
         FGui.SpBtSearchHyperLinksCurrentMemberClick(Self);
       end;
    5: begin
         if InputQuery('Class Locator','Enter class name:',aClassName) then
           if not FExpertFunctions.LocateClass(aClassName) then
             MessageDlg('Class "'+aClassName+'" could not be found', mtWarning, [mbOK], 0);
       end;
    6: begin
          with TDiagramGenerator.Create do
          begin
            GenerateDefaultFolderSet;
            Free;
          end;
       end;
    7: begin
         if assigned(MMToolServices.Navigator.CurClass) then
           currClassMember:=MMToolServices.Navigator.CurClass.Name;
         if assigned(MMToolServices.Navigator.CurMember) then
           currClassMember:=currClassMember+'.'+MMToolServices.Navigator.CurMember.Name;
         FGui.SequencesMemo.Lines.Add(currClassMember);
       end;
    8: begin
         FGui.Show;
         FGui.PgctrlMain.ActivePage:=FGui.tbsMethodRef;
         aRefList:=TStringList.Create;
         try
           FExpertFunctions.LookUpMethodReferences(aRefList);
           FGui.lbxMethodRef.Items.Assign(aRefList);
         finally
           aRefList.Free;
         end;
       end;
    else
      Inherited Execute(Index);
  end;
end;

function TQutronicExpertInterface.ExpertID: WideString;
begin
  Result := 'QutronicExperts';
end;

function TQutronicExpertInterface.GetMenuPositions(Index: Integer): 
        TMMMenuPosition;
begin
  case Index of
    0: Result := mpToolsMenu;
    1: Result := mpToolsMenu;
    2: Result := mpToolsMenu;
    3: Result := mpDiagramEditorMenu;
    4: Result := mpToolsMenu;
    5: Result := mpToolsMenu;
    6: Result := mpToolsMenu;
    7: Result := mpToolsMenu;
    8: Result := mpToolsMenu;
  end;
end;

function TQutronicExpertInterface.GetMenuShortCuts(Index: Integer): TShortCut;
begin
  case Index of
    0: Result := TextToShortCut('Alt+h'); // header
    1: Result := TextToShortCut('Alt+e'); // try except
    2: Result := TextToShortCut('Alt+q'); // QME show
    3: Result := TextToShortCut('Alt+d'); // diagram references
    4: Result := TextToShortCut('Alt+r'); // Member references
    5: Result := TextToShortCut('Alt+c'); // locate Class
    6: Result := TextToShortCut('Alt+f'); // add default Folderset
    7: Result := TextToShortCut('Alt+m'); // add current method to List
    8: Result := TextToShortCut('Alt+y'); // add Look up referencing methods
    else
    Result:=0;
  end;
end;

function TQutronicExpertInterface.GetVerbCount: Integer;
begin
  result:=9;
end;

function TQutronicExpertInterface.GetVerbs(Index: Integer): WideString;
begin
  case Index of
    0: Result := 'QMME -> Generate Header';
    1: Result := 'QMME -> Add Try/Except to current method';
    2: Result := 'QMME -> Show';
    3: Result := 'QMME -> Diagram References';
    4: Result := 'QMME -> Member References';
    5: Result := 'QMME -> Locate Class';
    6: Result := 'QMME -> Add Default Folders';
    7: Result := 'QMME -> Add Current Method to Seq. List';
    8: Result := 'QMME -> Look up referencing methods';
  end;
end;

procedure TQutronicExpertInterface.OnMbError(Sender : TObject ; Error : 
        TMailBoxErrors);
begin
end;

procedure TQutronicExpertInterface.OnMbReceive(Sender : TObject ; RxMessage : 
        TMemoryStream);
var
  aSeqGen: TDiagramGenerator;
  aSequence: TStringList;
begin
  RxMessage.Position:=0;
  aSequence:=TStringList.Create;
  aSequence.LoadFromStream(RxMessage);
  aSeqGen := TDiagramGenerator.Create;
  try
    aSeqGen.InvertSequence(aSequence);
    aSeqGen.GenSqDg(NIL,'sqDelphiCallStack',aSequence);
  finally
    aSeqGen.Free;
    aSequence.Free;
  end;
end;


end.
