{------------------------------------------------------------------------------

Copyright (C) 2002  Rainer Queck
e-mail: Rainer.Queck@Qutronic.de
http://www.qutronic.de


This code is free; you can redistribute it and/or
modify it under the terms of the MPL MOZILLA PUBLIC LICENSE. 

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
MPL MOZILLA PUBLIC LICENSE for more details at http://www.mozilla.org/MPL

//-----------------------------------------------------------------------------}
// Unit Created at    : Date: 16.02.02  Time :13:19:15
// Unit last modified : Date: 17.04.03  Time :08:14:49
//------------------------------------------------------------------------------
unit QutronicExpertClasses;

interface

uses Windows, SysUtils, Classes,Forms,Dialogs,extctrls,
     MMToolsApi,MMDiagramApi,MMEngineDefs,RQClasses;

type
  TRefType=(rtNone,rtMember,rtDiagram);
  TDiagramSpec=Record
    DiagramName : String;
    SymbolName  : String;
    Diagram     : IMMDiagram;
    DiagramId   : integer;
    Shape       : IMMShape;
  end;
  PDiagramSpec=^TDiagramSpec;
  TMMUnitExplorer = class (TObject)
  private
    FCode: TStringList;
    FTheUnit: IMMUnit;
    FUsesList: TStringList;
    procedure Analyze;
    procedure AnalyzeUses;
    procedure CodeToStringList;
    procedure ReplaceUses;
    procedure SetTheUnit(Value: IMMUnit);
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddUses(aUnit : String);
    procedure RemoveUses(aUnit : String);
    property TheUnit: IMMUnit read FTheUnit write SetTheUnit;
  end;
  
  TExpertFunctions = class (TObject)
  private
    FAliasManager: IMMAliasManager;
    FClass: IMMClass;
    FClassBase: IMMClassBase;
    FCodeModel: IMMCodeModel;
    FHeader: TStringList;
    FMethod: IMMMethod;
    FMMUnitExplorer: TMMUnitExplorer;
    FProjectManager: IMMProjectManager;
    FToolService: IMMToolServices;
    FUnit: IMMUnit;
    FUnitManager: IMMUnitManager;
    procedure CreateAField(Name,DataName: string; Visiability :TVisibility);
    procedure CreateAMethod(aName,aParameters : String; aMethodKind 
            :TMethodKind; aVisibility :TVisibility);
    procedure GetCurrentSelections;
    function ReferencesMethod(aMethod : IMMMethod; aMethodName : String): 
            Integer;
    procedure StringToStringList(aStr : String; aStrList : TStringList);
  protected
    function CreateAClass(aClassName : String): IMMClass;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddHeader;
    procedure AddToUses(aUnit : String);
    procedure ApplyTryExceptToCurrentMethod;
    function LocateClass(aClassName : String): Boolean;
    procedure LockupStringReference(var aRefList: TStringList ; aString : 
            String);
    procedure LookUpMethodReferences(var aRefList : TStringList);
  end;
  
  TMMDiagramExplorer = class (TObject)
  private
    FDiagnose: TDiagnose;
    FDiagramSelection: TList;
    FExMsg: string;
    procedure ClearDiagramSelection;
    function GetCurClassName: string;
    function GetCurDiagramName: string;
    function GetCurMemberName: string;
    function GetDiagramCount: Integer;
    function IsDiagramLink(aHyperLink : IMMSymbolLink): Boolean;
    function IsMemberOfCurClass(aHyperLink : IMMSymbolLink): Boolean;
    procedure LocateDiagramReferences(aRefStr : String; var aStrList, 
            SymbolNames : TStringList; Identical : Boolean ; aRefType : 
            TRefType);
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddLink;
    procedure AssignDiag(aDiag : TDiagnose);
    procedure GetCurrentDiagramReferences(var aStrList : TStringList; 
            SymbolNames : TStringList);
    procedure GetCurrentMemberReferences(var aStrList : TStringList; 
            SymbolNames : TStringList);
    function GetDiagramName(aIdx : Integer): string;
    procedure GoToDiagram(aIdx : Integer);
    procedure ValidateNames;
    property CurClassName: string read GetCurClassName;
    property CurDiagramName: string read GetCurDiagramName;
    property CurMemberName: string read GetCurMemberName;
    property DiagramCount: Integer read GetDiagramCount;
  end;
  

implementation

{
******************************* TMMUnitExplorer ********************************
}
constructor TMMUnitExplorer.Create;
begin
  inherited Create;
  FUsesList:=TStringList.Create;
  FCode:=TStringList.Create;
end;

destructor TMMUnitExplorer.Destroy;
begin
  FUsesList.Free;
  FCode.Free;
  inherited Destroy;
end;

procedure TMMUnitExplorer.AddUses(aUnit : String);
begin
  FUsesList.Add(aUnit);
  ReplaceUses;
end;

procedure TMMUnitExplorer.Analyze;
begin
  CodeToStringList;
  AnalyzeUses;
end;

procedure TMMUnitExplorer.AnalyzeUses;
var
  UsesStart: Integer;
  UsesEnd: Integer;
  aUsesLine: string;
  aPos: Integer;
  aUsedUnit: string;
  i: Integer;
  aLen: Integer;
begin
  UsesStart:=-1;
  UsesEnd:=-1;
  // find the USES lines
  for i:=0 to FCode.Count-1 do
  begin
    if Pos('uses',FCode.Strings[i])>0 then
      UsesStart:=i;
    if (UsesStart>=0) and (Pos(';',FCode.Strings[i])>0) then
    begin
      UsesEnd:=i;
      break;
    end;
  end;
  
  // add used units to UsesList
  FUsesList.Clear;
  for i:=UsesStart to UsesEnd do
  begin
    aUsesLine:=FCode.Strings[i];
    // remove the string 'uses' form line if it exists..
    aPos:=Pos('uses',aUsesLine);
    aLen:=Length('uses');
    if aPos>0 then
      Delete(aUsesLine,aPos,aLen);
    // now remove spaces from string
    aUsesLine:=Stringreplace(aUsesLine,' ','',[rfIgnoreCase,rfReplaceAll]);
    // get all used units...
    while (Pos(',',aUsesLine))>0 do
    begin
      aPos:=Pos(',',aUsesLine);
      // remove leading ','
      if aPos=1 then
      begin
        Delete(aUsesLine,aPos,1);
        aPos:=Pos(',',aUsesLine);
      end;
      aUsedUnit:=Copy(aUsesLine,1,aPos-1);
      FUsesList.Add(aUsedUnit);
      Delete(aUsesLine,1,aPos);
    end;
    // if there is a ';' then get the last used unit...
    if Pos(';',aUsesLine)>0 then
    begin
      aUsesLine:=Stringreplace(aUsesLine,';','',[rfIgnoreCase,rfReplaceAll]);
      FUsesList.Add(aUsesLine);
    end;
  end;
end;

procedure TMMUnitExplorer.CodeToStringList;
var
  i: Integer;
  CodeString: string;
begin
  CodeString:=String(FTheUnit.GetCode);
  FCode.Clear;
  i:=Pos(#$D#$A,CodeString);
  while i>0 do
  begin
    FCode.Add(Copy(CodeString,1,i-1));
    Delete(CodeString,1,i+1);
    i:=Pos(#$D#$A,CodeString);
  end;
  FCode.Add(CodeString);
end;

procedure TMMUnitExplorer.RemoveUses(aUnit : String);
var
  aIndex: Integer;
begin
  aIndex:=FUsesList.IndexOf(aUnit);
  if  aIndex>=0 then
  begin
    FUsesList.Delete(aIndex);
    ReplaceUses;
  end;
end;

procedure TMMUnitExplorer.ReplaceUses;
var
  UsesStart: Integer;
  UsesEnd: Integer;
  i: Integer;
  aIndex: Integer;
  LineCnt: Integer;
begin
  UsesStart:=-1;
  UsesEnd:=-1;
  // find the USES lines
  for i:=0 to FCode.Count-1 do
  begin
    if Pos('uses',FCode.Strings[i])>0 then
      UsesStart:=i;
    if (UsesStart>=0) and (Pos(';',FCode.Strings[i])>0) then
    begin
      UsesEnd:=i;
      break;
    end;
  end;
  // Delet existing uses...
  for i:=UsesStart to UsesEnd do
  begin
    FCode.Delete(UsesStart);
  end;
  // Prepare UsesList for replacing
  LineCnt:=0;
  FUsesList.Strings[LineCnt]:='uses '+FUsesList.Strings[LineCnt]+', ';
  for i:=1 to FUsesList.Count-1 do
  begin
    FUsesList.Strings[LineCnt]:=FUsesList.Strings[LineCnt]+FUsesList.Strings[i];
    if i<FUsesList.Count-1 then
    begin
      FUsesList.Strings[LineCnt]:=FUsesList.Strings[LineCnt]+', ';
    end
    else
    begin
      FUsesList.Strings[LineCnt]:=FUsesList.Strings[LineCnt]+';';
      break;
    end;
    if Length(FUsesList.Strings[LineCnt])>70 then
    begin
      LineCnt:=LineCnt+1;
      FUsesList.Strings[LineCnt]:='     ';
    end;
  end;
  for i:=LineCnt+1 to FUsesList.Count-1 do
  begin
    FUsesList.Delete(LineCnt+1);
  end;
  // Insert new uses
  aIndex:=UsesStart;
  for i:=0 to FUsesList.Count-1 do
  begin
    FCode.Insert(aIndex,FUsesList.Strings[i]);
    aIndex:=aIndex+1;
  end;
  // write code back to unit. and reanalyze
  FCode.SaveToFile('c:\temp\test.pas');
  FTheUnit.Code:=FCode.Text;
  Analyze;
end;

procedure TMMUnitExplorer.SetTheUnit(Value: IMMUnit);
begin
  FTheUnit:=Value;
  Analyze;
end;

{
******************************* TExpertFunctions *******************************
}
constructor TExpertFunctions.Create;
begin
  FHeader:=TStringList.Create;
  
  FToolService:=MMToolServices;
  FAliasManager:=FToolService.GetAliasManager;
  FUnitManager:=FToolService.UnitManager;
  FProjectManager:=FToolService.GetProjectManager;
  FCodeModel:=FToolService.GetCodeModel;
  GetCurrentSelections;
  FMMUnitExplorer:=TMMUnitExplorer.Create;
end;

destructor TExpertFunctions.Destroy;
begin
  FHeader.Free;
  FMMUnitExplorer.Free;
  inherited Destroy;
end;

procedure TExpertFunctions.AddHeader;
var
  HeaderFile: string;
begin
  FToolService.GetProjectManager.CommitEditors;
  HeaderFile:=ExtractFilePath(Application.Exename)+'header.txt';
  if FileExists(HeaderFile) then
  begin
    FHeader.LoadFromFile(HeaderFile);
  end
  else
  begin
    FHeader.Add('// no Headerfile found  : ' +HeaderFile) ;
  end;
  FHeader.Add('// Unit Created at    : Date: ' +DateToStr(now)+'  Time :'+TimeToStr(now));
  FHeader.Add('// Unit last modified : Date: '+#60+'!Date!'+#62+'  Time :'+#60+'!Time!'+#62+'');
  FHeader.Add('//------------------------------------------------------------------------------');
  FUnit:=FToolService.Navigator.GetCurUnit;
  FUnit.Code:=FHeader.GetText+FUnit.Code;
  
  
  // FMMUnitExplorer.TheUnit:=FUnit;
  // FMMUnitExplorer.RemoveUses('Hugo');
  // FToolService.GetProjectManager.CommitEditors;
end;

procedure TExpertFunctions.AddToUses(aUnit : String);
var
  CodeStrings: TStringList;
  i: Integer;
  UsesStart: Integer;
  UsesEnd: Integer;
  aString: string;
begin
  FProjectManager.CommitEditors;
  CodeStrings:=TStringList.Create;
  UsesStart:=-1;
  UsesEnd:=-1;
  try
    CodeStrings.SetText(PChar(FUnit.GetCode));
    StringToStringList(String(FUnit.GetCode),CodeStrings);
    // Find Uses Start and End;
    for i:=0 to CodeStrings.Count-1 do
    begin
      if (Pos('uses',CodeStrings[i])>0) then
      begin
        UsesStart:=i;
      end;
      if (UsesStart>0) and (Pos(';',CodeStrings[i])>0) then
      begin
        UsesEnd:=i;
        break;
      end;
    end;
    // Now create wanted uses clause....
    // ...,IB_Components,IB_Session;
    aString:=CodeStrings.Strings[UsesEnd];
    aString[Pos(';',aString)]:=',';
    CodeStrings.Strings[UsesEnd]:=aString;
    CodeStrings.Insert(UsesEnd+1,aUnit+';');
    FUnit.SetCode(CodeStrings.GetText);
  finally
    CodeStrings.Free;
  end;
end;

procedure TExpertFunctions.ApplyTryExceptToCurrentMethod;
var
  ASection: TStringList;
  aIndex: Integer;
  someCode: TStringList;
begin
  //=== Add new sections...
  GetCurrentSelections;
  ASection:=TStringList.Create;
  try
    ASection.Add('//=== TRY');
    ASection.Add('TRY');
    FMethod.InsertSection(0,WideString(ASection.Text));
  
    ASection.Clear;
    ASection.Add('//===');
    ASection.Add('EXCEPT');
    ASection.Add('  on E: Exception do');
    ASection.Add('  begin');
    ASection.Add('    FExMsg:='+#39+'Exception in '+#60+'!ClassName!>.'+#60+'!MemberName!>:'+#39+'+E.Message;');
    ASection.Add('    if FDiagnose<>nil then');
    ASection.Add('      FDiagnose.ReportError(0,FExMsg)');
    ASection.Add('    else');
    ASection.Add('      ShowMessage(FExMsg);');
    ASection.Add('  end;');
    ASection.Add('END;');
    FMethod.AddSection(WideString(ASection.Text));
  finally
    ASection.Free;
  end;
  //=== Add Diagnose Fields
  if not FClassBase.FindMember('FDiagnose',aIndex) then
  begin
    CreateAField('FDiagnose','TDiagnoseRq',scPrivate);
  end;
  if not FClassBase.FindMember('FExMsg',aIndex) then
  begin
    CreateAField('FExMsg','string',scPrivate);
  end;
  //=== Method to assign FDiagnoseRq
  someCode:=TStringList.Create;
  try
    if FClassBase.FindMember('AssignDiag',aIndex) then
      exit;
    // else lets create this method
    CreateAMethod('AssignDiag','aDiag : TDiagnoseRq',mkProcedure,scPublic);
    someCode.Clear;
    someCode.Add('FDiagnose:=aDiag;');
    FMethod.AddSection(WideString(someCode.Text));
  finally
    someCode.Free;
  end;
end;

function TExpertFunctions.CreateAClass(aClassName : String): IMMClass;
var
  Anc: IMMClass;
  i: Integer;
begin
  FClass:= MMToolServices.CodeModel.AddClass;
  FClass.Name :=aClassName;
  for i := MMToolServices.CodeModel.ClassCount - 1 downto 0 do
    if MMToolServices.CodeModel.Classes[I].QueryInterface(IMMClass, Anc) = S_OK then
    begin
      if Anc.Name='TObject' then
      begin
        FClass.Ancestor := Anc;
        Break;
      end;
    end;
  result:=FClass;
end;

procedure TExpertFunctions.CreateAField(Name,DataName: string; Visiability 
        :TVisibility);
var
  aField: IMMField;
begin
  aField:=FClassBase.AddField;
  aField.Name:=Name;
  aField.DataName:=DataName;
  aField.Visibility:=Visiability;
end;

procedure TExpertFunctions.CreateAMethod(aName,aParameters : String; 
        aMethodKind :TMethodKind; aVisibility :TVisibility);
begin
  FMethod:=FClassBase.AddMethod;
  FMethod.Name:=aName;
  FMethod.MethodKind:=aMethodKind;
  FMethod.Parameters:=aParameters;
  FMethod.Visibility:=aVisibility;
  if aMethodKind=mkDestructor then
  begin
    FMethod.BindingKind:=bkOverride;
  end;
end;

procedure TExpertFunctions.GetCurrentSelections;
begin
  FMethod:=MMToolServices.GetNavigator.GetCurMethod;
  FClassBase:=MMToolServices.GetNavigator.GetCurClass;
  FUnit:=MMToolServices.GetNavigator.GetCurUnit;
end;

function TExpertFunctions.LocateClass(aClassName : String): Boolean;
var
  i: Integer;
  aClass: IMMClassBase;
  n: Integer;
  aMember: IMMMember;
begin
  result:=False;
  //=== check if a class with this name exists
  for i:=0 to MMToolServices.CodeModel.ClassCount-1 do
  begin
    aClass:=MMToolServices.CodeModel.Classes[i];
    if UpperCase(aClass.Name)=UpperCase(aClassName) then
    begin
      MMToolServices.Navigator.CurClass:=aClass;
      result:=True;
      exit;
    end;
  end;
  
  //=== if no class with this name exists lest see if a member with that name exists.
  for i:=0 to MMToolServices.CodeModel.ClassCount-1 do
  begin
    aClass:=MMToolServices.CodeModel.Classes[i];
    for n:=0 to aClass.MemberCount-1 do
    begin
      aMember:=aClass.Members[n];
      if UpperCase(aMember.Name)=UpperCase(aClassName) then
      begin
          if LocateClass(aMember.DataName) then
          begin
            Result:=True;
          end;
      end;
    end;
  end;
end;

procedure TExpertFunctions.LockupStringReference(var aRefList: TStringList ; 
        aString : String);
var
  i: Integer;
  aClass: IMMClassBase;
  k: Integer;
  ReferencingSection: Integer;
begin
  aRefList.Clear;
  aRefList.Add('<<'+aString+'>>');
  for i:=0 to MMToolServices.CodeModel.ClassCount-1 do
  begin
    aClass:=MMToolServices.CodeModel.Classes[i];
    for k:=0 to aClass.MemberCount-1 do
    begin
      if aClass.Members[k].MemberType=cpMethod then
      begin
        ReferencingSection:=ReferencesMethod(IMMMethod(aClass.Members[k]),aString);
        if ReferencingSection>0 then
        begin
          aRefList.Add(aClass.Name+'.'+aClass.Members[k].Name+' Section:'+IntToStr(ReferencingSection));
        end;
      end;
    end;
  end;
end;

procedure TExpertFunctions.LookUpMethodReferences(var aRefList : TStringList);
var
  aMethodName: string;
  i: Integer;
  aClass: IMMClassBase;
  k: Integer;
  ReferencingSection: Integer;
begin
  aRefList.Clear;
  aMethodName:=MMToolServices.GetNavigator.CurMethod.Name;
  aRefList.Add('<<'+MMToolServices.GetNavigator.CurClass.Name+
               '.'+aMethodName+'>>');
  for i:=0 to MMToolServices.CodeModel.ClassCount-1 do
  begin
    aClass:=MMToolServices.CodeModel.Classes[i];
    for k:=0 to aClass.MemberCount-1 do
    begin
      if aClass.Members[k].MemberType=cpMethod then
      begin
        ReferencingSection:=ReferencesMethod(IMMMethod(aClass.Members[k]),aMethodName);
        if ReferencingSection>0 then
        begin
          aRefList.Add(aClass.Name+'.'+aClass.Members[k].Name+' Section:'+IntToStr(ReferencingSection));
        end;
      end;
    end;
  end;
end;

function TExpertFunctions.ReferencesMethod(aMethod : IMMMethod; aMethodName : 
        String): Integer;
var
  i: Integer;
  SectionText: string;
begin
  Result:=0;
  for i:=0 to aMethod.SectionCount-1 do
  begin
    SectionText:=aMethod.Sections[i];
    if Pos(aMethodName,SectionText)>0 then
    begin
      Result:=i+1;
      exit;
    end;
  end;
end;

procedure TExpertFunctions.StringToStringList(aStr : String; aStrList : 
        TStringList);
var
  i: Integer;
  l: Integer;
begin
  { Modelmakers wide string...:
  'unit RQExpertInterface;'#$D#$A#$D#$A'interface'#$D#$A#$D#$A'uses SysUtils, Classes;'#$D#$A#$D#$A'implementation'#$D#$A#$D#$A'end.'#$D#$A
  }
  aStrList.Clear;
  i:=Pos(#$D#$A#$D#$A,aStr);
  l:=sizeof(#$D#$A#$D#$A);
  while i>0 do
  begin
    aStrList.Add(Copy(aStr,1,i-1));
    aStrList.Add('');
    aStr:=Copy(aStr,i+l,Length(aStr)-i+l);
    i:=Pos(#$D#$A#$D#$A,aStr);
  end;
  aStrList.Add(aStr);
end;

{
****************************** TMMDiagramExplorer ******************************
}
constructor TMMDiagramExplorer.Create;
begin
  inherited Create;
  
  
  FDiagramSelection := TList.Create;
end;

destructor TMMDiagramExplorer.Destroy;
begin
  FDiagramSelection.Free;
  
  inherited Destroy;
end;

procedure TMMDiagramExplorer.AddLink;
var
  MySymbol: IMMSymbol;
  DiagramExplorer: IMMDiagramExplorer;
  Diagram: IMMDiagram;
  DiagramSpec: PDiagramSpec;
  i: Integer;
  PackageSymbol: IMMSymbol;
begin
  //=== TRY
  TRY
  
  Diagram:=MMToolServices.GetNavigator.GetCurDiagram;
  DiagramExplorer:=Diagram.Explore as IMMDiagramExplorer;
  PackageSymbol:=DiagramExplorer.CreateSymbol('PackageSymbol',NIL);
  PackageSymbol.Location:=Point(20,20);
  PackageSymbol.Width:=200;
  PackageSymbol.Height:=30+20*FDiagramSelection.Count;
  PackageSymbol.Name:='called by:';
  DiagramExplorer.Commit;
  
  
  for i:=0 to FDiagramSelection.Count-1 do
  begin
    DiagramSpec:=FDiagramSelection.Items[i];
    Diagram:=MMToolServices.GetNavigator.GetCurDiagram;
    DiagramExplorer:=Diagram.Explore as IMMDiagramExplorer;
    MySymbol:=DiagramExplorer.CreateSymbol('DiagramLink',PackageSymbol);
    MySymbol.Location:=Point(10,10+20*(i+1));
    MySymbol.HotlinkToEntity(nttDiagram,DiagramSpec.Diagram.Id);
    DiagramExplorer.Commit;
  end;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.AddLink:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

procedure TMMDiagramExplorer.AssignDiag(aDiag : TDiagnose);
begin
  FDiagnose:=aDiag;
end;

procedure TMMDiagramExplorer.ClearDiagramSelection;
var
  i: Integer;
  aDiagramSpec: PDiagramSpec;
begin
  //=== TRY
  TRY
  
  for i:=0 to FDiagramSelection.Count-1 do
  begin
    aDiagramSpec:=FDiagramSelection.Items[i];
    Dispose(aDiagramSpec);
    FDiagramSelection.Items[i]:=NIL;
  end;
  FDiagramSelection.Pack;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.ClearDiagramSelection:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

function TMMDiagramExplorer.GetCurClassName: string;
begin
  Result:=MMToolServices.GetNavigator.GetCurClass.Name;
end;

function TMMDiagramExplorer.GetCurDiagramName: string;
begin
  //=== TRY
  TRY
  
  Result:='';
  if assigned(MMToolServices.GetNavigator.GetCurDiagram) then
    Result:=MMToolServices.GetNavigator.GetCurDiagram.Name;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.GetCurDiagramName:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

function TMMDiagramExplorer.GetCurMemberName: string;
begin
  //=== TRY
  TRY
  
  Result:=MMToolServices.GetNavigator.GetCurMember.Name;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.GetCurMemberName:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

procedure TMMDiagramExplorer.GetCurrentDiagramReferences(var aStrList : 
        TStringList; SymbolNames : TStringList);
begin
  //=== TRY
  TRY
  
  LocateDiagramReferences(CurDiagramName,aStrList,SymbolNames,True,rtDiagram);
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.GetCurrentDiagramReferences:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

procedure TMMDiagramExplorer.GetCurrentMemberReferences(var aStrList : 
        TStringList; SymbolNames : TStringList);
begin
  //=== TRY
  TRY
  
  LocateDiagramReferences(CurMemberName,aStrList,SymbolNames,True,rtMember);
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.GetCurrentMemberReferences:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

function TMMDiagramExplorer.GetDiagramCount: Integer;
begin
  Result:=MMToolServices.GetDiagramManager.DiagramCount;
end;

function TMMDiagramExplorer.GetDiagramName(aIdx : Integer): string;
var
  DiagramSpec: PDiagramSpec;
begin
  //=== TRY
  TRY
  
  result:='';
  
  if aIdx<FDiagramSelection.Count then
  begin
    DiagramSpec:=FDiagramSelection.Items[aIdx];
    result:=DiagramSpec.DiagramName;
  end;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.GetDiagramName:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

procedure TMMDiagramExplorer.GoToDiagram(aIdx : Integer);
var
  DiagramSpec: PDiagramSpec;
begin
  //=== TRY
  TRY
  
  if aIdx<FDiagramSelection.Count then
  begin
    DiagramSpec:=FDiagramSelection.Items[aIdx];
    MMToolServices.GetNavigator.SetCurDiagram(DiagramSpec.Diagram);
    MMToolServices.Navigator.ActivateView(vtDiagramEditor,True);
  end;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.GoToDiagram:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

function TMMDiagramExplorer.IsDiagramLink(aHyperLink : IMMSymbolLink): Boolean;
var
  aDiagram: IMMDiagram;
  EntityReference: IUnknown;
begin
  //=== TRY
  result:=False;
  TRY
  
  EntityReference:=aHyperlink.EntityRef;
  
  if EntityReference.QueryInterface(IMMDiagram,aDiagram)=S_OK then
  begin
      Result:=True;
  end;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.IsDiagramLink:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

function TMMDiagramExplorer.IsMemberOfCurClass(aHyperLink : IMMSymbolLink): 
        Boolean;
var
  EntityReference: IUnknown;
  aMember: IMMMember;
begin
  //=== TRY
  result:=False;
  TRY
  
  EntityReference:=aHyperlink.EntityRef;
  
  if EntityReference.QueryInterface(IMMMember,aMember)=S_OK then
  begin
    if aMember.ClassBase.Name=CurClassName then
    begin
      Result:=True;
    end;
  end;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.IsMemberOfCurClass:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

procedure TMMDiagramExplorer.LocateDiagramReferences(aRefStr : String; var 
        aStrList, SymbolNames : TStringList; Identical : Boolean ; aRefType : 
        TRefType);
var
  DiagramManager: IMMDiagramManager;
  Diagram: IMMDiagram;
  DiagramExplorer: IMMDiagramExplorer;
  Shape: IMMShape;
  i: Integer;
  n: Integer;
  k: Integer;
  NextDiagram: Boolean;
  Hyperlink: IMMSymbolLink;
  Location: string;
  DiagramSpec: PDiagramSpec;
begin
  //=== TRY
  TRY
  
      ClearDiagramSelection;
      DiagramManager:=MMToolServices.GetDiagramManager;
      for i:=0 to DiagramManager.DiagramCount-1 do
      begin
        NextDiagram:=False;
        Diagram:=DiagramManager.GetDiagrams(i);
        DiagramExplorer:=IMMDiagramExplorer(Diagram.Explore);
        for n:=0 to DiagramExplorer.ShapeCount-1 do
        begin
          Shape:=DiagramExplorer.Shapes[n];
          for k:=0 to Shape.HyperlinkCount-1 do
          begin
            Hyperlink:=Shape.Hyperlinks[k];
            Location:=Hyperlink.Location;
            if Identical then
            begin
              if Location=aRefStr then
              begin
                case aRefType of
                  rtMember: begin
                              if IsMemberOfCurClass(HyperLink) then
                              begin
                                New(DiagramSpec);
                                DiagramSpec.DiagramName:=Diagram.GetName;
                                DiagramSpec.SymbolName:=Shape.Name;
                                DiagramSpec.Diagram:=Diagram;
                                DiagramSpec.DiagramId:=Diagram.GetId;
                                DiagramSpec.Shape:=Shape;
                                FDiagramSelection.Add(DiagramSpec);
                                NextDiagram:=True;
                                break;
                              end;
                            end;
                  rtDiagram : begin
                                if IsDiagramLink(HyperLink) then
                                begin
                                  New(DiagramSpec);
                                  DiagramSpec.DiagramName:=Diagram.GetName;
                                  DiagramSpec.SymbolName:=Shape.Name;
                                  DiagramSpec.Diagram:=Diagram;
                                  DiagramSpec.Shape:=Shape;
                                  FDiagramSelection.Add(DiagramSpec);
                                  NextDiagram:=True;
                                  break;
                                end;
                              end;
                end;
              end;
            end
            else
            begin
              raise Exception.Create('aRefStr in Location not done jet');
            end;
          end;
          if NextDiagram then
            break;
        end;
      end;
  
  aStrList.Clear;
  SymbolNames.Clear;
  // add found diagrams to aStrList
  for i:=0 to FDiagramSelection.Count-1 do
  begin
    DiagramSpec:=FDiagramSelection.Items[i];
    aStrList.Add(DiagramSpec.DiagramName);
    SymbolNames.Add(DiagramSpec.SymbolName);
  end;
  
  //===
  EXCEPT
    on E: Exception do
    begin
      FExMsg:='Exception in TMMDiagramExplorer.LocateDiagramReferences:'+E.Message;
      if FDiagnose<>nil then
        FDiagnose.Report('E',FExMsg)
      else
        ShowMessage(FExMsg);
    end;
  END;
end;

procedure TMMDiagramExplorer.ValidateNames;
var
  i: Integer;
  Diagram: IMMDiagram;
  
  procedure CheckName(PreFix : String);
  var
    aPos : Integer;
    aStr : String;
  begin
    aPos:=Pos(' ',Diagram.Name);
    while aPos>0 do
    begin
      aStr:=Diagram.Name;
      aPos:=Pos(' ',aStr);
      aStr[aPos+1]:=Char(Ord(aStr[aPos+1])AND $DF);
      Delete(aStr,aPos,1);
      Diagram.Name:=aStr;
    end;
    aPos:=Pos(PreFix,Diagram.Name);
    if (aPos>1) or (aPos=0) then
    begin
      Diagram.Name:=PreFix+Diagram.Name;
    end;
  end;
  
begin
  for i:=0 to MMToolServices.DiagramManager.DiagramCount-1 do
  begin
    Diagram:=MMToolServices.DiagramManager.Diagrams[i];
    case Diagram.DiagramType of
      diClassDiagram         : CheckName('cs');
      diSequenceDiagram      : CheckName('sq');
      diUseCaseDiagram       : CheckName('uc');
      diUnitDependencyDiagram: CheckName('ud');
      diActivityDiagram      : CheckName('ac');
      diStateDiagram         : CheckName('st');
      diImplementationDiagram: CheckName('im');
      diMindMapDiagram       : CheckName('mm');
      diCollaborationDiagram : CheckName('cl');
      diRobustnessDiagram    : CheckName('rb');
    end;
  end;
end;


end.
