{------------------------------------------------------------------------------

Copyright (C) 2002  Rainer Queck
e-mail: Rainer.Queck@Qutronic.de
http://www.qutronic.de


This code is free; you can redistribute it and/or
modify it under the terms of the MPL MOZILLA PUBLIC LICENSE. 

This code is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
MPL MOZILLA PUBLIC LICENSE for more details at http://www.mozilla.org/MPL

//-----------------------------------------------------------------------------}
// Unit Created at    : Date: 15.11.02  Time :17:20:20
// Unit last modified : Date: 17.04.03  Time :08:15:30
//------------------------------------------------------------------------------
unit TypeStream;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, TypInfo;

type
  TStreamObject = class;
  TTypeMemoryStream = class (TMemoryStream)
  public
    function ReadBoolean: Boolean;
    function ReadByte: Byte;
    function ReadDouble: Double;
    function ReadInt64: Int64;
    function ReadInteger: Integer;
    procedure ReadList(aList : TList;  aClassType: TClass);
    function ReadLongInt: LongInt;
    function ReadLongWord: LongWord;
    procedure ReadObject(aStreamObject : TStreamObject);
    function ReadString: string;
    function ReadWord: Word;
    procedure WriteBoolean(Value : Boolean);
    procedure WriteByte(Value : Byte);
    procedure WriteDouble(Value : Double);
    procedure WriteInt64(Value : Int64);
    procedure WriteInteger(Value : Integer);
    procedure WriteList(aList : TList);
    procedure WriteLongInt(Value : LongInt);
    procedure WriteLongWord(Value : LongWord);
    procedure WriteObject(aStreamObject : TStreamObject);
    procedure WriteString(Value : String);
    procedure WriteWord(Value : Word);
  end;
  
  TStreamObject = class (TObject)
  public
    procedure LoadFromStream(aStream : TTypeMemoryStream); virtual; abstract;
    procedure SaveToStream(aStream : TTypeMemoryStream); virtual; abstract;
  end;
  

procedure Register;

implementation

procedure Register;
begin
end;

{
****************************** TTypeMemoryStream *******************************
}
function TTypeMemoryStream.ReadBoolean: Boolean;
var
  aByte: Byte;
begin
  Read(aByte,SizeOf(aByte));
  if aByte=0 then
   Result:=False
  else
   Result:=True;
end;

function TTypeMemoryStream.ReadByte: Byte;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

function TTypeMemoryStream.ReadDouble: Double;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

function TTypeMemoryStream.ReadInt64: Int64;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

function TTypeMemoryStream.ReadInteger: Integer;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

procedure TTypeMemoryStream.ReadList(aList : TList;  aClassType: TClass);
var
  ObjectCount: Integer;
  i: Integer;
  aStreamObject: TStreamObject;
begin
  // setzt voraus, das die Liste Objekte mit LoadFrom-/SaveToFile Methoden hat!
  
  // Achtung: Methode ist noch nicht getestet! RQ 19.11.02
  ObjectCount:=ReadInteger;
  for i:=0 to ObjectCount-1 do
  begin
    aStreamObject:=TStreamObject(aClassType.Create);
    ReadObject(aStreamObject);
    aList.Add(aStreamObject);
  end;
end;

function TTypeMemoryStream.ReadLongInt: LongInt;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

function TTypeMemoryStream.ReadLongWord: LongWord;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

procedure TTypeMemoryStream.ReadObject(aStreamObject : TStreamObject);
begin
  try
    aStreamObject.LoadFromStream(self);
  except
    on E: Exception do
        raise Exception.Create(E.ClassName+' in TTypeMemoryStream.ReadObject : Object is probably not Streamable; '+E.Message);
  
  end;
end;

function TTypeMemoryStream.ReadString: string;
var
  aStringSize: Integer;
begin
  Result:='';
  Read(aStringSize,SizeOf(aStringSize));
  SetLength(Result,aStringSize);
  Read(Result[1],aStringSize);
end;

function TTypeMemoryStream.ReadWord: Word;
begin
  Result:=0;
  Read(Result,SizeOf(Result));
end;

procedure TTypeMemoryStream.WriteBoolean(Value : Boolean);
var
  aByte: Byte;
begin
  if Value then
   aByte:=1
  else
   aByte:=0;
  
  Write(aByte,SizeOf(aByte));
end;

procedure TTypeMemoryStream.WriteByte(Value : Byte);
begin
  Write(Value,SizeOf(Value));
end;

procedure TTypeMemoryStream.WriteDouble(Value : Double);
begin
  Write(Value,SizeOf(Value));
end;

procedure TTypeMemoryStream.WriteInt64(Value : Int64);
begin
  Write(Value,SizeOf(Value));
end;

procedure TTypeMemoryStream.WriteInteger(Value : Integer);
begin
  Write(Value,SizeOf(Value));
end;

procedure TTypeMemoryStream.WriteList(aList : TList);
var
  ObjectCount: Integer;
  i: Integer;
  aStreamObject: TStreamObject;
begin
  // setzt voraus, das die Liste Objekte mit LoadFrom-/SaveToFile Methoden hat!
  
  // Achtung: Methode ist noch nicht getestet! RQ 19.11.02
  ObjectCount:=aList.Count;
  WriteInteger(ObjectCount);
  for i:=0 to ObjectCount-1 do
  begin
    aStreamObject:=TStreamObject(aList.Items[i]);
    WriteObject(aStreamObject);
  end;
end;

procedure TTypeMemoryStream.WriteLongInt(Value : LongInt);
begin
  Write(Value,SizeOf(Value));
end;

procedure TTypeMemoryStream.WriteLongWord(Value : LongWord);
begin
  Write(Value,SizeOf(Value));
end;

procedure TTypeMemoryStream.WriteObject(aStreamObject : TStreamObject);
begin
  try
    aStreamObject.SaveToStream(self);
  except
    on E: Exception do
        raise Exception.Create(E.ClassName+' in TTypeMemoryStream.WriteObject : Object is probably not Streamable; '+E.Message);
  
  end;
end;

procedure TTypeMemoryStream.WriteString(Value : String);
var
  aStringSize: Integer;
begin
  aStringSize:=Length(Value);
  Write(aStringSize,SizeOf(aStringSize));
  Write(Value[1],aStringSize);
end;

procedure TTypeMemoryStream.WriteWord(Value : Word);
begin
  Write(Value,SizeOf(Value));
end;


end.
